import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import React, { Component } from 'react';
import TranslateMaker from 'translate-maker';
import { mount } from 'enzyme';
import Translate, { ProvideTranslate, LocaleProvider, Namespace, LocaleSwitch } from './index';

describe('Translate', () => {
  it('should be able to create simple instance', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          test: 'Test response'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'test' })
    ));

    expect(wrapper.text()).toBe('Test response');
  }));

  it('should be able to create simple instance with default value', () => {
    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK' },
      React.createElement(Translate, { path: 'test', defaultValue: 'Default value' })
    ));

    expect(wrapper.text()).toBe('Default value');
  });

  it('should be able to create simple instance with default value as children', () => {
    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK' },
      React.createElement(
        Translate,
        { path: 'test' },
        'Default value'
      )
    ));

    expect(wrapper.text()).toBe('Default value');
  });

  it('should be able to create element h1', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          test: 'Test response'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(
        'h1',
        null,
        React.createElement(Translate, { path: 'test', defaultValue: 'Default value' })
      )
    ));

    expect(wrapper.html()).toBe('<h1>Test response</h1>');
  }));

  it('should be able to use variable inside translation', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          hello: 'Hello {$name}'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'hello', name: 'Zlatko' })
    ));

    expect(wrapper.text()).toBe('Hello Zlatko');
  }));

  it('should be able to use plurals inside translation', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          followers: `{$name} has {$followers, plural,
            =0 {no followers}
            =1 {# follower}
               {# followers}
          }`
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'followers', name: 'Zlatko', followers: 0 })
    ));

    expect(wrapper.text()).toBe('Zlatko has no followers');
  }));

  it('should be able to use params instead of props', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          hello: 'Hello {$name}'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const user = {
      name: 'Zlatko'
    };

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'hello', params: user })
    ));

    expect(wrapper.text()).toBe('Hello Zlatko');
  }));

  it('should be able to use html content', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          hello: 'Welcome back <b>{$name}</b>. How is it going?'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'hello', name: 'Zlatko', html: true })
    ));

    expect(wrapper.html()).toBe('<span>Welcome back <b>Zlatko</b>. How is it going?</span>');
  }));

  it('should be able to use filters', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          hello: 'Hello {$name | star}'
        }
      },
      filters: {
        star: function (value) {
          return '*** ' + value + ' ***';
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'hello', name: 'Zlatko' })
    ));

    expect(wrapper.text()).toBe('Hello *** Zlatko ***');
  }));

  it('should be able to use namespace', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          namespace: {
            test: 'Test response'
          }
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(
        Namespace,
        { path: 'namespace' },
        React.createElement(Translate, { path: 'test' })
      )
    ));

    expect(wrapper.text()).toBe('Test response');
  }));

  it('should be able to change namespace', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          namespace1: {
            test: 'Test response'
          },
          namespace2: {
            test: 'Test response2'
          }
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const Main = function ({ namespace }) {
      return React.createElement(
        LocaleProvider,
        { locale: 'sk_SK', translate: translate },
        React.createElement(
          Namespace,
          { path: namespace },
          React.createElement(Translate, { path: 'test' })
        )
      );
    };

    const wrapper = mount(React.createElement(Main, { namespace: 'namespace1' }));
    expect(wrapper.text()).toBe('Test response');

    wrapper.setProps({ namespace: 'namespace2' });
    expect(wrapper.text()).toBe('Test response2');
  }));

  it('should be able to use namespace inside namespace', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          namespace1: {
            namespace2: {
              test: 'Test response'
            }
          }
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(
        Namespace,
        { path: 'namespace1' },
        React.createElement(
          Namespace,
          { path: 'namespace2', compose: true },
          React.createElement(Translate, { path: 'test' })
        )
      )
    ));

    expect(wrapper.text()).toBe('Test response');
  }));

  it('should be able to use namespace inside namespace with replace', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          namespace: {
            test: 'Test response'
          }
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(
        Namespace,
        { path: 'namespace1' },
        React.createElement(
          Namespace,
          { path: 'namespace' },
          React.createElement(Translate, { path: 'test' })
        )
      )
    ));

    expect(wrapper.text()).toBe('Test response');
  }));

  it('should be able to use LocaleSwitch as uncontrolled component', _asyncToGenerator(function* () {
    const data = {
      sk_SK: {
        test: 'Testovacia odpoved'
      },
      en_US: {
        test: 'Test response'
      }
    };

    const translate = new TranslateMaker({
      data
    });

    yield translate.setLocale('sk_SK');

    let changeLocale;

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { data: data, locale: 'sk_SK', translate: translate, controlled: false },
      React.createElement(
        'div',
        null,
        React.createElement(
          'span',
          null,
          React.createElement(Translate, { path: 'test' })
        ),
        React.createElement(
          LocaleSwitch,
          {
            onError: function (err) {
              throw err;
            }
          },
          function (props) {
            changeLocale = props.changeLocale;
            return null;
          }
        )
      )
    ));

    expect(wrapper.text()).toBe('Testovacia odpoved');

    yield changeLocale('en_US');

    expect(wrapper.text()).toBe('Test response');
  }));

  it('should be able to use LocaleSwitch as uncontrolled component', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          test: 'Testovacia odpoved'
        },
        en_US: {
          test: 'Test response'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    return new Promise(function (resolve, reject) {
      let wrapper;

      function handleLocaleChange(newLocale) {
        expect(newLocale).toBe('en_US');
        expect(wrapper.find('span').html()).toBe('<span>Test response</span>');
        resolve();
      }

      let onChange;

      const Main = function () {
        return React.createElement(
          LocaleProvider,
          { translate: translate, controlled: false },
          React.createElement(
            'div',
            null,
            React.createElement(
              'span',
              null,
              React.createElement(Translate, { path: 'test' })
            ),
            React.createElement(
              LocaleSwitch,
              {
                onChange: handleLocaleChange,
                onError: reject
              },
              function ({ changeLocale }) {
                onChange = changeLocale;
                return null;
              }
            )
          )
        );
      };

      wrapper = mount(React.createElement(Main, null));

      expect(wrapper.find('span').html()).toBe('<span>Testovacia odpoved</span>');

      onChange('en_US');
      // wrapper.find('select').simulate('change', {target: { value : 'en_US'}});
    });
  }));

  it('should be able to use t decorator', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          test: 'Testovacia odpoved'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    class Test extends Component {
      render() {
        return React.createElement(
          'div',
          null,
          this.props.translate('test')
        );
      }
    }

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(
        ProvideTranslate,
        null,
        function (t) {
          return React.createElement(Test, { translate: t });
        }
      )
    ));

    expect(wrapper.html()).toBe('<div>Testovacia odpoved</div>');
  }));

  it('should be able to other components as props', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          test: 'Testovacia odpoved {$link}'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'test', link: React.createElement(
          'a',
          null,
          'Asdf'
        ), html: true })
    ));

    expect(wrapper.html()).toBe('<span>Testovacia odpoved <a>Asdf</a></span>');
  }));

  it('should be able to other components as props with Translate', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          test: 'Testovacia odpoved {$link}',
          inner: '123'
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', translate: translate },
      React.createElement(Translate, { path: 'test', link: React.createElement(
          'a',
          null,
          'Asdf ',
          React.createElement(Translate, { path: 'inner' })
        ), html: true })
    ));

    expect(wrapper.html()).toBe('<span>Testovacia odpoved <a>Asdf 123</a></span>');
  }));

  it('should be able to use namespace', _asyncToGenerator(function* () {
    const translate = new TranslateMaker({
      data: {
        sk_SK: {
          namespace: {
            test: 'Testovacia odpoved'
          }
        }
      }
    });

    yield translate.setLocale('sk_SK');

    const wrapper = mount(React.createElement(
      LocaleProvider,
      { locale: 'sk_SK', namespace: 'namespace', translate: translate },
      React.createElement(
        Namespace,
        { path: 'namespace' },
        React.createElement(Translate, { path: 'test', html: true })
      )
    ));

    expect(wrapper.html()).toBe('<span>Testovacia odpoved</span>');
  }));
});
//# sourceMappingURL=Translate.test.js.map